@@ -94,25 +94,23 @@ class Lexer(object):
         if DEBUG:
             print('response ok? %s (responseCode=%x), error-code: %x, message size: %d' % \
                 (self.responseOK, self.responseCode,  self.errCode,  self.messageSize))
+        # store total size of message in case buffer needs to be emptied after an error occurred:
+        self.totalMessageSize = self.lexpos + self.messageSize
         return self.messageSize
 
     def clearSocketData(self):
         '''
         If for any reason the parsing process returns an error, make sure that all data from
         a socket is removed to avoid data pollution with further parsing attempts.
-        This should only be called after self.readHeader() has been executed.
+        This must only be called after self.readHeader() has been executed.
         '''
-        if type(self.fp) != socket.socket:  #hasattr(self.fp, '_sock'):
-            # probably not a socket. Nothing to do here.
+        if not isinstance(self.fp, socket.socket):
+            # not a socket. Nothing to do here.
             return
-        self.fp.setblocking(0)
-        try:
-            while 1:
-                self.fp.recv(SOCKET_BLOCK_SIZE)
-        except:
-            pass
-        finally:
-            self.fp.setblocking(1)
+        bytesLeft = self.totalMessageSize - self.lexpos
+        while bytesLeft > 0:
+            d = self.fp.recv(SOCKET_BLOCK_SIZE)
+            bytesLeft -= len(d)
 
     def read(self, length):
         '''
@@ -146,6 +144,9 @@ class Lexer(object):
         if tCode == XT_INT3:
             length = 3
             rawData = self.read(length) + b'\x00'
+        elif tCode == XT_INT7:
+            length = 7
+            rawData = self.read(length) + b'\x00'
         else:
             length = struct.calcsize(fmt or 1)
             rawData = self.read(length)
@@ -160,12 +161,18 @@ class Lexer(object):
         - an REXPR header
         '''
         startLexpos = self.lexpos
-        _rTypeCode  = self.__unpack('B') # unsigned byte!
-        rTypeCode   =  _rTypeCode & (0xFF - XT_HAS_ATTR)   # remove XT_HAS_ATTR flag (if it exists)
-        hasAttr     = (_rTypeCode & XT_HAS_ATTR) != 0      # extract XT_HAS_ATTR flag (if it exists)
-        length      = self.__unpack(XT_INT3)
-        if not rTypeCode in VALID_R_TYPES:
-            raise RParserError("Invalid token %s found at lexpos %d, length %d" % 
+        _rTypeCode  = self.__unpack('B')  # unsigned byte!
+        rTypeCode   =  _rTypeCode & 0x3F                 # extract pure rTypeCode without XT_HAS_ATTR or XT_LARGE flags
+        hasAttr     = (_rTypeCode & XT_HAS_ATTR) != 0    # extract XT_HAS_ATTR flag (if it exists)
+        isXtLarge   = (_rTypeCode & XT_LARGE)    != 0    # extract XT_LARGE flag (if it exists)
+        if isXtLarge:
+            # header is larger, use all 7 bytes for length information (new in Rserve 0.3)
+            length  = self.__unpack(XT_INT7)
+        else:
+            # small header, use 3 bytes for length information
+            length  = self.__unpack(XT_INT3)
+        if rTypeCode not in VALID_R_TYPES:
+            raise RParserError("Unknown SEXP type %s found at lexpos %d, length %d" %
                                (hex(rTypeCode), startLexpos, length))
         return Lexeme(rTypeCode, length, hasAttr, startLexpos)
         
@@ -260,14 +267,13 @@ class RParser(object):
     parserMap = {}
     fmap = FunctionMapper(parserMap)
     #
-    def __init__(self, src, atomicArray, arrayOrder):
+    def __init__(self, src, atomicArray):
         '''
         atomicArray: if False parsing arrays with only one element will just return this element
         arrayOrder:  The order in which data in multi-dimensional arrays is returned. 'C' for c-order, F for fortran.
         '''
         self.lexer = Lexer(src)
         self.atomicArray = atomicArray
-        self.arrayOrder  = arrayOrder
 
     def __getitem__(self, key):
         return self.parserMap[key]
@@ -287,9 +293,8 @@ class RParser(object):
         if DEBUG:
             l = lexeme
             typeCodeDict = XTs if isRexpr else DTs
-            print('%s %s (%s), hasAttr=%s, lexpos=%d, length=%s' % \
-                  (self.__ind, typeCodeDict[l.rTypeCode], hex(l.rTypeCode),
-                   l.hasAttr, l.lexpos, l.length))
+            print('%s %s (%s), hasAttr=%s, lexpos=%d, length=%s' %
+                  (self.__ind, typeCodeDict[l.rTypeCode], hex(l.rTypeCode), l.hasAttr, l.lexpos, l.length))
 
     def parse(self):
         '''
@@ -318,7 +323,8 @@ class RParser(object):
         dataLexeme = self.lexer.nextExprHdr()
         self._debugLog(dataLexeme, isRexpr=False)
         if dataLexeme.rTypeCode == DT_SEXP:
-            return self._postprocessData(self._parseExpr().data)
+            expression = self._parseExpr()
+            return self._postprocessData(expression.data)
         else:
             raise NotImplementedError()
 
@@ -369,27 +375,6 @@ class RParser(object):
                     data = str(data)      # convert into native python string
                 elif isinstance(data, (numpy.bool_, numpy.bool, numpy.bool8)):
                     data = bool(data)     # convert into native python string
-
-            elif len(data.shape) > 1 and self.arrayOrder == 'F':
-                # Convert array into fortran-style ordering.
-                #
-                # Multi-dimensional arrays can have different data ordering: c-style or fortran-style.
-                # In the former case the last index moves fastest, in fortran style it is the first index.
-                # See numpy docs for further details.
-                # Example:
-                # >>> conn.r("matrix(1:6, nrow=2, ncol=3)")  with 'C' ordering would result in
-                # array([[1, 2, 3],
-                #        [4, 5, 6]])
-                #
-                # while conn.r("matrix(1:6, nrow=2, ncol=3)")  with 'F' ordering would result in
-                # array([[1, 3, 5],
-                #        [2, 4, 6]])
-                #
-                # By default all arrays are at 'C'-style at this point. To convert it into fortran style
-                # the reshape() function has to be applied. To make this work the array must be made 1-d first!!!
-                shape = data.shape[::-1]               # save the shape information, just backwards
-                data.shape = data.size                 # make it one-dimensional
-                data = data.reshape(shape, order='F')  # now reshape, using the saved shape info, into fortran order
         return data
 
     @fmap(None)
@@ -399,12 +384,12 @@ class RParser(object):
 
     @fmap(XT_ARRAY_BOOL, XT_ARRAY_INT, XT_ARRAY_DOUBLE, XT_ARRAY_STR)
     def xt_array(self, lexeme):
-        data = self._nextExprData(lexeme)  # converts lexeme into a numpy array
+        data = self._nextExprData(lexeme)  # converts data into a numpy array already
         if lexeme.hasAttr and lexeme.attrTypeCode == XT_LIST_TAG:
             for tag, value in lexeme.attr:
                 if tag == 'dim':
-                    # the array has a defined shape
-                    data.shape = value
+                    # the array has a defined shape, and R stores and sends arrays in Fortran mode:
+                    data = data.reshape(value, order='F')
                 elif tag == 'names':
                     # convert numpy-vector 'value' into list to make TaggedArray work properly:
                     data = asTaggedArray(data, list(value))
@@ -417,7 +402,7 @@ class RParser(object):
                         data = asAttrArray(data, {tag: value})
         return data
 
-    @fmap(XT_VECTOR, XT_LANG_NOTAG, XT_LIST_NOTAG)
+    @fmap(XT_VECTOR, XT_VECTOR_EXP, XT_LANG_NOTAG, XT_LIST_NOTAG)
     def xt_vector(self, lexeme):
         '''
         A vector is e.g. return when sending "list('abc','def')" to R. It can contain mixed
@@ -427,6 +412,9 @@ class RParser(object):
         is known in advance.
         The end of this REXP can only be detected by keeping track of how many bytes
         have been consumed (lexeme.length!) until the end of the REXP has been reached.
+
+        A vector expression (type 0x1a) is according to Rserve docs the same as XT_VECTOR. For now
+        just a list with the expression content is returned in this case.
         '''
         finalLexpos = self.lexer.lexpos + lexeme.dataLength
         if DEBUG:
@@ -476,8 +464,8 @@ class RParser(object):
 
 ########################################################################################
 
-def rparse(src, atomicArray=False, arrayOrder='C'):
-    rparser = RParser(src, atomicArray, arrayOrder)
+def rparse(src, atomicArray=False):
+    rparser = RParser(src, atomicArray)
     return rparser.parse()
 
 ########################################################################################
